#include "stdafx.h"
#include "Packet.h"
#include "InstallInstallModule.h"
#include "IPC/IPCCopyData.h"
#include "Registry.h"
#include "util.h"
#include <shellapi.h>
#include "SetupBuilderManager.h"
#include "LauncherManager.h"
#include "AgentDLL.h"

using namespace tipbase;

const TCHAR *IDS_TIP_INSTALL_TEMP_DIRECTORY = _T("%s\\Ntalk biz");

class MyEventCallback : public IIPCReceiveCallback
{
public:
	MyEventCallback() : m_packetType(PacketCommand_Install), m_status(0)
	{
		m_handle = ::CreateEvent( NULL, TRUE, FALSE, NULL );
	}

	int Callback(IPCPacket& packet, IIPCHandle *callback_handle )
	{
		ExecuteStatusPacket *pHeader = (ExecuteStatusPacket *)packet.data;

		m_packetType	= pHeader->type;
		m_status		= pHeader->status;
		packet.Release();

		SetEvent( m_handle );
		return 1;
	}

	int	GetStatus()
	{
		return m_status;
	}

	HANDLE GetHandle()
	{
		return m_handle;
	}

	DWORD GetCommand()
	{
		return m_packetType;
	}

private:
	DWORD		m_packetType;
	HANDLE		m_handle;
	int			m_status;
};

class ProgramInstaller
{
public:
	ProgramInstaller( const TCHAR *installer_path, const TCHAR *install_dir, int execute_option )
	{
		m_path				= installer_path;
		m_dir				= install_dir;
		m_execute_option	= execute_option;
	}

	CString GetTempPathForExtract()
	{
		TCHAR szFilename[1024];
		TCHAR szTempDir[1024];
		TCHAR szTempSubDir[1024];

		Checked::tcscpy_s( szFilename, 1024, ATLPath::FindFileName( m_path ) );

		GetTempPath( 1024, szTempDir );
		wsprintf(szTempSubDir, IDS_TIP_INSTALL_TEMP_DIRECTORY, szTempDir);
		CreateDirectory( szTempSubDir, NULL );

		ATLPath::Append( szTempSubDir, szFilename );

		return CString(szTempSubDir);
	}

	BOOL Extract()
	{
		m_SubPath = GetTempPathForExtract();
		return setupBuilder.ExtractSourceFile(m_path,m_SubPath);
	}

	BOOL Install()
	{
		MyEventCallback myEventCallback;
		
		IIPCHandle* launcherHandle = LauncherManager::GetInstance()->GetLauncherHandle();
		if( NULL == launcherHandle )
		{
				return FALSE;
		}

		launcherHandle->SetReadCallback( &myEventCallback );

		ExecutePacket command;
		command.type = PacketCommand_Install;
		command.execute_option = m_execute_option;

		if( m_path.IsEmpty() == FALSE ){
			// [:2008-09-30]    
			_tcscpy_s( command.installPath, tipbase::MAXPATH, m_dir );
		}
		else
			command.installPath[0] = 0;

		_tcscpy_s( command.path, tipbase::MAXPATH, m_SubPath );
		if (!PathFileExists(command.path))
		{
			_tcscpy_s( command.path, tipbase::MAXPATH, m_path );
		}
		launcherHandle->Write( (const BYTE *)&command, sizeof(ExecutePacket));
		
		if( FALSE == WaitForHandleTipLauncher( myEventCallback.GetHandle(), TIPLAUNCHER_CHECK_TIME ) )
			return FALSE;

		launcherHandle->SetReadCallback(NULL);
		if( myEventCallback.GetCommand() != PacketCommand_ExecuteStatus )
		{
			return FALSE;
		}

		if( myEventCallback.GetStatus() != 0 )
		{
			return FALSE;
		}

		return TRUE;
	}

	void PostScript()
	{
		setupBuilder.PostScript();
	}

private:
	CString		m_SubPath;
	CString		m_path;
	CString		m_dir;
	SetupBuilderManager setupBuilder;

	int			m_execute_option;
};

BOOL InstallProgram( LPCTSTR path, LPCTSTR installDir, int execute_option )
{
	ProgramInstaller installer( path, installDir, execute_option );
	if( FALSE == installer.Extract() )
	{
		return FALSE;
	}

	if( FALSE == installer.Install() )
	{
		return FALSE;
	}

	installer.PostScript();
	return TRUE;
}

ERRORINFO *MakeErrorInfo( int code, const TCHAR *msg )
{
	ERRORINFO *errorInfo = new ERRORINFO;
	if( NULL == errorInfo )
		return NULL;

	errorInfo->code = code;
	errorInfo->message = (TCHAR *)msg;
	if( msg )
	{
		int len = lstrlen(msg);
		errorInfo->message = new TCHAR[len+1];
		Checked::tcscpy_s( errorInfo->message, len+1, (const TCHAR *)msg );
	}

	return errorInfo;
}

BOOL InstallInstallModule::Install()
{
	GetCallback()->OnStart();
	CALLBACK_FINISHED_STATUS status = CALLBACK_FINISHED_SUCCESS;
	ERRORINFO *errorInfo = NULL;

	if( FALSE == InstallProgram( m_path.c_str(), m_installDir.c_str(), m_execute_option ) )
		status = CALLBACK_FINISHED_ERROR;

	AgentDLLWrapper *agentDLL = AgentDLLManager::Find( GetGuid() );
	if( agentDLL )
	{
		CString msg = _T("");
		int code = -1;

		if( TRUE == agentDLL->GetInstallError( &code, &msg ) )
		{		
			errorInfo = MakeErrorInfo(code, (const TCHAR *)msg );
		}
	}

	if( errorInfo )
		status = CALLBACK_FINISHED_ERROR;

	SetModuleStatus(ModuleStatusFINISHED);
	GetCallback()->OnFinished(status, errorInfo);
	return TRUE;
}

// BOOL InstallInstallModule::Install()
// {
// #ifdef INSTALL_CONTROLLER_FAKE
// 	_sleep(5000);
// 	while( IsPending() == TRUE )
// 	{
// 		_sleep(0);
// 	}
// 	SetModuleStatus(ModuleStatusFINISHED);
// 	GetCallback()->OnFinished(CALLBACK_FINISHED_SUCCESS);
// 	return TRUE;
// #endif
// 	TCHAR tempName[1024];
// 	TCHAR szTempPath[1024];
// 	TCHAR szTempSubPath[1024];
// 
// 	Checked::tcscpy_s( tempName, 1024, ATLPath::FindFileName( m_path.c_str() ) );
// 
// 	GetTempPath( 1024, szTempPath );
// 	wsprintf(szTempSubPath, IDS_TIP_INSTALL_TEMP_DIRECTORY, szTempPath);
// 	CreateDirectory( szTempSubPath, NULL );
// 
// 	ATLPath::Append( szTempSubPath, tempName );
// 	ExecutePacket command;
// 	CALLBACK_FINISHED_STATUS status = CALLBACK_FINISHED_SUCCESS;
// 
// 	SetupBuilderManager setupBuilder;
// 
// 	if( FALSE == setupBuilder.ExtractSourceFile(m_path.c_str(),szTempSubPath) )
// 	{
// 		SetError();
// 		goto InstallInstallModule_Error;
// 	}
// 
// 	IIPCHandle* launcherHandle = LauncherManager::GetInstance()->GetLauncherHandle();
// 	if( NULL == launcherHandle )
// 	{
// 		SetError();
// 		goto InstallInstallModule_Error;
// 	}
// 
// 	MyEventCallback myEventCallback; 
// 	launcherHandle->SetReadCallback( &myEventCallback );
// 
// 	command.type = PacketCommand_Execute;
// 	command.silent = this->m_silent;
// 
// 	if( m_path.empty() == false )
// 		// [:2008-09-30]    
// 		_tcscpy_s( command.installPath, tipbase::MAXPATH, m_installDir.c_str()/*m_path.c_str()*/ );
// 	else
// 		command.installPath[0] = 0;
// 	
// 	_tcscpy_s( command.path, tipbase::MAXPATH, szTempSubPath );
// 
// 	launcherHandle->Write( (const BYTE *)&command, sizeof(ExecutePacket));
// 
// 	IPCPacket packet;
// 	
// 	while( launcherHandle->Read(packet) == FALSE )
// 	{
// 		_sleep(0);
// 	}
// 
// 	ProtocolPacketHeader *pHeader = (ProtocolPacketHeader *)packet.data;
// 	
// 	if( PacketCommand_ExecuteStatus != pHeader->type )
// 	{
// 		SetError();
// 	}
// 	else
// 	{
// 		setupBuilder.PostScript();
// 	}
// 
// InstallInstallModule_Error:	
// 	while( IsPending() == TRUE )
// 	{
// 		_sleep(0);
// 	}
// 
// 	SetModuleStatus(ModuleStatusFINISHED);
// 	if( IsError() )
// 		status = CALLBACK_FINISHED_ERROR;
// 
// 	GetCallback()->OnFinished(status);
// 	return TRUE;
// }